#ifndef CoCoA_RandomSource_H
#define CoCoA_RandomSource_H

//   Copyright (c)  2010  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------

#include "CoCoA/MachineInteger.H"
#include "CoCoA/ZZ.H"

#include "gmp.h"
#include <iosfwd>
//using std::ostream;

namespace CoCoA
{

  class RandomSource
  {
  public:
    explicit RandomSource(const MachineInteger& seed = 0);
    explicit RandomSource(const ZZ& seed);
    RandomSource(const RandomSource& copy);
    RandomSource& operator=(const RandomSource& rhs);
    ~RandomSource();
    friend bool RandomBool(RandomSource& src);
    friend long RandomLong(RandomSource& src);
    friend long RandomLong(RandomSource& src, const MachineInteger& lwb, const MachineInteger& upb);
    friend ZZ RandomZZ(RandomSource& src, const MachineInteger& lwb, const MachineInteger& upb);
    friend ZZ RandomZZ(RandomSource& src, const MachineInteger& lwb, const ZZ& upb);
    friend ZZ RandomZZ(RandomSource& src, const ZZ& lwb, const MachineInteger& upb);
    friend ZZ RandomZZ(RandomSource& src, const ZZ& lwb, const ZZ& upb);
//    friend ZZ RandomZZmod(RandomSource& src, const ZZ& M);  // range is 0..M-1
    friend void reseed(RandomSource& src, const MachineInteger& seed);
    friend void reseed(RandomSource& src, const ZZ& seed);
    friend std::ostream& operator<<(std::ostream& out, const RandomSource& src);
  private:  // data members
    ZZ mySeed;
    gmp_randstate_t myState;
//     long myBoolCounter;
//     long myLongCounter;
//     long myZZCounter;
  };


  inline bool RandomBool(RandomSource& src)
  {
    return gmp_urandomb_ui(src.myState, 1) == 1;
  }


} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/RandomSource.H,v 1.3 2011/05/03 09:52:50 abbott Exp $
// $Log: RandomSource.H,v $
// Revision 1.3  2011/05/03 09:52:50  abbott
// Changed default initial seed to 0 (was previsouly 1).
// Added reseeding functions.
// Seeding from a ZZ is now possible.
//
// Revision 1.2  2011/03/14 10:26:22  abbott
// Changed size_t into long.
//
// Revision 1.1  2010/12/17 16:03:15  abbott
// Added new class RandomSource.
//
//

#endif
