#ifndef CoCoALibSupplement_H
#define CoCoALibSupplement_H

//   Copyright (c)  2011  Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include <vector>
#include "CoCoA/factorization.H"

namespace CoCoA
{
// fwd decl -- defined in library.H
	class ConstMatrixView;
	class ConstRefRingElem;
	class MatrixView;
	class QQ;
	class RingElem;
	class RingHom;
	class ZZ;
	class ideal;
	class matrix;
	class ring;

  std::vector<long> VectorLong(const std::vector<ZZ>& ZZVec, const std::string& FuncName);
  
  RingElem LPP_forC5(ConstRefRingElem f); ///< NB result belongs to owner(f)
  RingElem LM_forC5(ConstRefRingElem f);  ///< NB result belongs to owner(f)
  bool IsTerm_forC5(ConstRefRingElem f);
  RingElem LT_forC5(ConstRefRingElem f); ///< NB result belongs to owner(f)
  RingElem CoeffOfTerm_forC5(ConstRefRingElem f, ConstRefRingElem t); ///< NB result belongs to CoeffRing(owner(f))
  std::vector<RingElem> coefficients_forC5(ConstRefRingElem f);  ///< NB result belongs to CoeffRing(owner(f))
  std::vector<RingElem> monomials_forC5(ConstRefRingElem f);  ///< NB result belongs to owner(f)
  std::vector<RingElem> support_forC5(ConstRefRingElem f);  ///< NB result belongs to owner(f)

  RingElem DensePoly_forC5(const ring& P, const ZZ& D);

  std::vector<ZZ> wdeg_forC5(ConstRefRingElem f);
  ZZ NextPrime_forC5(ZZ N);
  factorization<long> SmoothFactor_forC5(const ZZ& n, const ZZ& TrialLimit);
  QQ CpuTime_forC5();
  std::string date_forC5();
  
  RingElem lcm_forC5(const std::vector<RingElem>& v);
  RingElem gcd_forC5(const std::vector<RingElem>& v);  

  RingElem HilbertNumQuot_forC5(const ideal& I);

  //----- matrix --------------------------------------------------
  matrix NewDenseMat_forC5(const ring& R, const ZZ& NR, const ZZ& NC);
  matrix LexMat_forC5(const ZZ& N);  
  matrix RevLexMat_forC5(const ZZ& N);
  matrix StdDegLexMat_forC5(const ZZ& N);
  matrix StdDegRevLexMat_forC5(const ZZ& N);
  matrix XelMat_forC5(const ZZ& N);  
  matrix ExtractOrdMat_forC5(const matrix& M);
  void SetEntry_forC5(MatrixView& M, const ZZ& I, const ZZ& J, ConstRefRingElem x);
  std::vector<RingElem> GetRow_forC5(ConstMatrixView M, const ZZ& I);
  std::vector<RingElem> GetCol_forC5(ConstMatrixView M, const ZZ& I);
  void SetRow_forC5(MatrixView& M, const ZZ& I, const std::vector<RingElem>& v);
  void SetCol_forC5(MatrixView& M, const ZZ& I, const std::vector<RingElem>& v);
  MatrixView IdentityMat_forC5(const ring& R, const ZZ& N);
  MatrixView ZeroMat_forC5(const ring& R, const ZZ& NRows, const ZZ& NCols);
  MatrixView FilledMat_forC5(const ring& R, const ZZ& NRows, const ZZ& NCols, ConstRefRingElem x);
  ConstMatrixView transposed_forC5(ConstMatrixView M);

  long GradingDim_forC5(const ring& P); // P must be SparsePolyRing
  matrix WeightsMatrix_forC5(const ring& P); // P must be SparsePolyRing
  matrix ord_forC5(const ring& P); // P must be SparsePolyRing

  //---- Normaliz ----  
  std::vector<std::vector<ZZ> > HilbertBasisNormaliz_forC5(const std::vector<std::vector<ZZ> >& v);
  
}
#endif // CoCoALibSupplement_H

